#!/bin/bash

set -e

KIND=$1
BITS=$2
REGS=$3
SHORT=$4

SIMDIR=sim/b$BITS

#echo "KIND:     $KIND"
#echo "BITS:     $BITS"
#echo "REGMASKS: $REGS"

case $SHORT
in
    short)
	PATTERN="_1.txt"
	;;
    long)
	PATTERN="-v _1.txt"
	;;
    *)
	echo "Bad short/long marker."
	exit 1;
	;;
esac

# First grep is to find only example files, second to choose which to use.
TRACES=`find traces -type f | grep .txt | grep $PATTERN | sort`

CNT=0

for i in $TRACES
do
    CNT=$((CNT+1))
    
    DIRNAME=$(dirname $i)
    BASENAME=$(basename $i .txt)

    TEMPFILES=

    OUTBASEC=$SIMDIR/out/$DIRNAME/$BASENAME

    if [ "$KIND" = "C" ]
    then
	# C simulation

	echo -n -e "                                                                               \r"
	echo -n -e "SIM-C-TEST: $OUTBASEC  \r"

	mkdir -p $(dirname $OUTBASEC)

	$SIMDIR/tb_c < $i > $OUTBASEC.c.raw || \
	    (cat $OUTBASEC.c.raw ; \
	     echo "SIM-C-FAIL: $SIMDIR/tb_c < $i > $OUTBASEC.c.raw     " ; exit 1)

	cat $OUTBASEC.c.raw | egrep "^OUT" > $OUTBASEC.c.out
	#TEMPFILES="$TEMPFILES $OUTBASEC.c.raw" # $OUTBASEC.c.out"
	rm $OUTBASEC.c.raw
    fi

    if [ "$KIND" = "VHDL" ]
    then
	# VHDL simulation

	for seed in 0 1
	do
	    for r in $REGS
	    do
		OUTBASEV=$SIMDIR/r$r/s$seed/out/$DIRNAME/$BASENAME

		echo -n -e "                                                                               \r"
		echo -n -e "SIM-TEST: $OUTBASEV  \r"

		mkdir -p $(dirname $OUTBASEV)

		OUTBASEVTMP=`mktemp`

		$SIMDIR/r$r/tb_dptc -gseed=$((seed * CNT)) \
		    --stop-time=5000000ns --vcd=$OUTBASEVTMP.vcd < $i \
		    2>&1 > \
			 $OUTBASEVTMP.out_err || \
		    (echo "SIM-FAIL: $OUTBASEV     " ; \
		     mv $OUTBASEVTMP.out_err $OUTBASEV.out_err ; \
		     mv $OUTBASEVTMP.vcd     $OUTBASEV.vcd ; \
		     exit 1)

		cat $OUTBASEVTMP.out_err | \
		    egrep -v "@(0ms|10ns|20ns|30ns|40ns).*NUMERIC_STD.(TO_INTEGER|\"=\"|\"\+\"|.)|SEED:" \
			  > $OUTBASEVTMP.out

		diff -u $OUTBASEC.c.out $OUTBASEVTMP.out || \
		    (echo "SIM-DIFF-FAIL: $OUTBASEV     " ; \
		     mv $OUTBASEVTMP.out_err $OUTBASEV.out_err ; \
		     mv $OUTBASEVTMP.out     $OUTBASEV.out ; \
		     mv $OUTBASEVTMP.vcd     $OUTBASEV.vcd ; \
		     exit 1)

		# mv $OUTBASEVTMP.out_err $OUTBASEV.out_err
		# mv $OUTBASEVTMP.out     $OUTBASEV.out
		# mv $OUTBASEVTMP.vcd     $OUTBASEV.vcd

		rm -f $OUTBASEVTMP \
		   $OUTBASEVTMP.out_err $OUTBASEVTMP.out $OUTBASEVTMP.vcd

		# TEMPFILES="$TEMPFILES $OUTBASEV.out $OUTBASEV.vcd"
	    done
	done
    fi

    # If we come here, all simulations succeeded and matched.  Delete the
    # temporary files.

    # rm $TEMPFILES

done

echo "SIM-$KIND-TEST $SIMDIR-r$REGS: done.                                    "
