#!/usr/bin/perl

use strict;
use warnings;

if (!($ARGV[0] =~ /^(.*)_[ks](.*)_r(.*)$/))
{ die "Bad target_regs argument ($ARGV[0])."; }

my $target = $1;
my $kind   = $2;
my $regs   = $3;

sub build($);

{
    my $timec = 20.0;

  trybuild:
    while (1)
    {
	my $minperiod = build($timec);

	if ($minperiod >= $timec) {
	    last trybuild;
	}

	$timec = sprintf("%.4f",$minperiod - 0.25);
    }   
}

sub build($)
{
    my $timec = shift;

    my $command =
	"TARGET=$target REGS=$regs KIND=$kind TIMEC=$timec ".
	"../tb_timing/build.sh";

    my $minperiod = 100;
    my $luts = 10000;
    my $ffs = 10000;
    my $srpfile = "-";

    open(my $fh, '-|', $command) or die $!;
    while (my $line = <$fh>) {
	# Do stuff with each $line.
	if ($line =~ /Minimum period:\s*([\d\.]*)\s*ns/) {
	    $minperiod = $1;
	}
	if ($line =~ /Number of.*LUTs:\s*([\d]*)\s*out/) {
	    $luts = $1;
	}
	if ($line =~ /Number of.*Flip Flops:\s*([\d]*)\s*out/) {
	    $ffs = $1;
	}
	if ($line =~ /Number of.*Registers:\s*([\d]*)\s*out/) {
	    $ffs = $1;
	}
	if ($line =~ /\.srp:\s(.*)/) {
	    $srpfile = $1;
	}
	if ($line =~ /\.sta.rpt:\s(.*)/) {
	    $srpfile = $1;
	}

	if ($line =~ /;\s*([\d\.]*)\s*MHz\s*;.*;\s*clk\s*;/) {
	    my $period = 1000.0 / $1;
	    #print " --- $1 -> $period";
	    if ($minperiod == 100 ||
		$period > $minperiod) {
		$minperiod = $period;
	    }
	}
	if ($line =~ /Total logic elements\s*:\s*([\d]*)\s*\//) {
	    $luts = $1;
	}
	if ($line =~ /Logic utilization.*:\s*([\d]*)\s*\//) {
	    $luts = $1;
	}
	if ($line =~ /Total registers.*:\s*([\d]*)\s*/) {
	    $ffs = $1;
	}
    }
    close $fh or die $!;

    print sprintf("%-20s %3.2b %10.6b %7.3f %4d %4d  %s\n",
		  $target,$kind,$regs,$minperiod,$luts,$ffs,$srpfile);

    return $minperiod;
}
