#!/usr/bin/perl

use strict;
use warnings;

my @entries = ();
my $tgt = ();
my $knd = ();

while (my $line = <>)
{
    if ($line =~ /([A-Za-z0-9]*)\s*([01]*)\s*([01]*)\s*([0-9\.]*)\s*([0-9]*)\s*([0-9]*)\s*(.*)/)
    {
	my $target  = $1;
	my $kind    = $2;
	my $regs    = $3;
	my $period  = $4;
	my $luts    = $5;
	my $ffs     = $6;
	my $srpfile = $7;

	push @entries,{"target"  => $target,
		       "regs"    => $regs,
		       "kind"    => $kind,
		       "period"  => $period,
		       "luts"    => $luts,
		       "ffs"     => $ffs,
		       "srpfile" => $srpfile};

	$tgt = $target;
	$knd = $kind;
    }
    # print ":: $line \n";
}

if (!$tgt || !$knd)
{
    die "Did not figure out tgt or knd.";
}

my @faster = sort { $a->{"period"} <=> $b->{"period"} } @entries;

print "=========================\n";
print "*** $tgt $knd ***\n\n";

print "Period   LUTs   FFs    Regs\n";
print "------   ----   ----   ------\n";

my $best = ();
my @shown = ();

foreach my $e (@faster)
{
    if (!$best ||
	$e->{"luts"} < $best->{"luts"})
    {
	if ($#shown > -1) { print "\n"; }
	print sprintf ("%-7.3f  %-4d   %-4d  ",
		       $e->{"period"},$e->{"luts"},$e->{"ffs"});

	$best = $e;
	@shown = ();
    }
    if ($e->{"period"} == $best->{"period"} &&
	$e->{"luts"} == $best->{"luts"})
    {
	foreach my $s (@shown)
	{
	    if ($s->{"regs"} eq $e->{"regs"}) { goto noshow; }
	}
	print sprintf (" %-6s",$e->{"regs"});
	print sprintf (" # %s",$e->{"srpfile"});
	push @shown, $e;
      noshow:
	;
    }
}

print "\n";
print "=========================\n";

